/*******************************************************************************
* Copyright Reticular Systems Inc. 1997    All rights reserved.
* Proprietary Information, Property of Reticular Systems Inc.
* Filename : BuyerFrame.java
*
* RCS: $Revision: 1.7 $  $Date: 1998/06/26 00:03:04 $  $Author: sandoval $ 
* Created by: Jim Donohoe -- Tue Sep  9 16:44:59 1997
*******************************************************************************/

package com.reticular.agents.buyerSeller;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

import com.reticular.agentBuilder.agent.perception.*;

/*******************************************************************************
* Description : Frame provides a simple interface for the message testing
* agent. This interface has a scrollable text area, a print button, and a quit
* button.
*
* 
* @author Jim Donohoe
*******************************************************************************/
public class BuyerFrame extends Frame
   implements Runnable, Cloneable, ActionListener
{ 

//PUBLIC DATA
   public static final int FRAME_WIDTH = 800;
   public static final int FRAME_HEIGHT = 600;
      
   public static final int INV_TEXT_AREA_ROWS = 5;
   public static final int INV_TEXT_AREA_COLS = 48;
   public static final int MSSG_TEXT_AREA_ROWS = 8;
   public static final int MSSG_TEXT_AREA_COLS = 48;
   public static final int QUANT_TEXT_FIELD_COLS = 8;
   public static final String EMPTY_STRING = "";
   public static final String SHOP_BUTTON_LABEL = " Shop! ";
   public static final String QUIT_BUTTON_LABEL =  " Quit ";

   public static final String PRODUCT_CHOICE_LABEL =  "Product ";
   public static final String QUANTITY_LABEL = "    Quantity:";
   public static final String INVENTORY_LABEL = "Inventory:";
   public static final String ACCOUNT_LABEL = "Account:";

   public static final String BUTTON_FONT_NAME = "SansSerif";
   public static final int BUTTON_FONT_STYLE = Font.BOLD;
   public static final int BUTTON_FONT_SIZE = 18;

   public static final String TEXT_FONT_NAME = "SansSerif";
   public static final int TEXT_FONT_STYLE = Font.PLAIN;
   public static final int TEXT_FONT_SIZE = 16;


//PRIVATE DATA
   private Choice _productChoice;
   private TextField _quantityTextField;

   private TextArea _messageTextArea;
   private TextArea _inventoryTextArea;
   private TextField _accountTextField;

   private Hashtable _productTable;
   private Hashtable _inventoryTable;
   private PacCommSystem _pacCommSystem;
   private String _buyerName;
   

   
/********************************PUBLIC METHODS********************************


/**
  * Description : 
  * 
  *   
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public static void main( String[] args )
   {
      BuyerFrame b = new BuyerFrame( null, "whatever");

      new Thread( (Runnable) b ).start();

      SellerFrame s = new SellerFrame( "Fred's Store" );

      s.run();
   }

                                 

/**
  * Description : BuyerFrame constructor
  *  
  * @param pacCommSystem - PacCommSystem for communication between the agent
  * and the PAC.
  * @param buyerName - String
  * @return   
  * @exception 
  * @author Jim Donohoe
  */
   public  BuyerFrame( PacCommSystem pacCommSystem, String buyerName )
   {
      // Store the pac comm system and buyer name into this object.
      _pacCommSystem = pacCommSystem;
      _buyerName = buyerName;
      
      // Initialize the product and inventory tables to empty hashtables.
      _productTable = new Hashtable();
      _inventoryTable = new Hashtable();

      // Initialize the components which may be need before the interface is
      // fully built.
      _productChoice = new Choice();
      _productChoice.add( PRODUCT_CHOICE_LABEL );

      _inventoryTextArea = new TextArea( EMPTY_STRING,
                                         INV_TEXT_AREA_ROWS,
                                         INV_TEXT_AREA_COLS,
                                         TextArea.SCROLLBARS_VERTICAL_ONLY );

      _accountTextField = new TextField( EMPTY_STRING );
   }



/**
  * Description : Append the string argument to the text area.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayMessage( String string )
   {
      _messageTextArea.append( string + "\n");
   }


/**
  * Description : Append the string argument to the text area.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayPriceQuote( PriceQuote quote )
   {
      _messageTextArea.append( quote.getStoreName() + ": " +
                               quote.getProductName() + " " +
                               quote.getPrice() + "\n");
   }   


/**
  * Description : Append the string argument to the text area.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayInventory( Product product )
   {
      // We need to keep the previous records in a table, so that we can
      // show the whole inventory each time we update it with a new entry.

      String itemName = product.getName();
      
      _inventoryTable.remove( itemName );

      _inventoryTable.put( itemName, product );

      // Clear the display area.
      _inventoryTextArea.setText("");

      Enumeration records = _inventoryTable.elements();
      while( records.hasMoreElements() )
      {
         try
         {
            Product prod = (Product) records.nextElement();

            String recordString = prod.getName() + ": " +
               Integer.toString( prod.getQuantity() ) + " " +
               prod.getUnitName() + "(s)\n";

            _inventoryTextArea.append( recordString );
         }
         catch( NoSuchElementException e )
         {
         }
      }
   }      


/**
  * Description : Convert the float to a string and display it in the
  * text field.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayAccount( float accountValue )
   {
      _accountTextField.setText( "$ " + Float.toString( accountValue ));
   }      

   
/**
  * Description : Adds an item to the choice
  * 
  *   
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void addProduct( Product product )
   {
      String productName = product.getName();
      _productTable.put( productName, product );
      
      _productChoice.addItem( productName );
   }

   
   
/**
  * Description : 
  * 
  *   
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void run( )
   {
      //-----------------------------------------------------------------------
      //            Set up the frame layout and common fonts
      //-----------------------------------------------------------------------
   
      GridBagLayout frameLayout = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      this.setLayout( frameLayout );

      Font textFont = new Font( TEXT_FONT_NAME, TEXT_FONT_STYLE,
                                TEXT_FONT_SIZE );

      Font buttonFont = new Font( BUTTON_FONT_NAME, BUTTON_FONT_STYLE,
                                  BUTTON_FONT_SIZE );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------

      _productChoice.setFont( buttonFont );
      
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 0;
      frameLayout.setConstraints( _productChoice, constraints );
      this.add( _productChoice );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------

      Label quantityLabel = new Label( QUANTITY_LABEL, Label.RIGHT );
      quantityLabel.setFont( buttonFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 1;
      constraints.gridy = 0;
      constraints.anchor = GridBagConstraints.EAST;
      frameLayout.setConstraints( quantityLabel, constraints );
      this.add( quantityLabel );
      
      _quantityTextField = new TextField( EMPTY_STRING,
                                          QUANT_TEXT_FIELD_COLS );

      _quantityTextField.setEditable( true );
      _quantityTextField.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 2;
      constraints.gridy = 0;
      constraints.anchor = GridBagConstraints.WEST;
      frameLayout.setConstraints( _quantityTextField, constraints );
      this.add( _quantityTextField );
      
     
      Button shopButton = new Button( SHOP_BUTTON_LABEL );
      shopButton.setFont( buttonFont );
      shopButton.setActionCommand( SHOP_BUTTON_LABEL );
      shopButton.addActionListener( this );
      constraints = new GridBagConstraints();
      constraints.gridx = 3;
      constraints.gridy = 0;
      constraints.anchor = GridBagConstraints.NORTHEAST;
      frameLayout.setConstraints( shopButton, constraints );
      this.add( shopButton );


      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------
      _messageTextArea = new TextArea( EMPTY_STRING,
                                       MSSG_TEXT_AREA_ROWS,
                                       MSSG_TEXT_AREA_COLS,
                                       TextArea.SCROLLBARS_VERTICAL_ONLY );

      _messageTextArea.setEditable( false );
      _messageTextArea.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 1;
      constraints.gridwidth = 3;
      constraints.gridheight = 2;
      frameLayout.setConstraints( _messageTextArea, constraints );
      this.add( _messageTextArea );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------
      Label inventoryLabel = new Label( INVENTORY_LABEL, Label.LEFT );
      inventoryLabel.setFont( buttonFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 3;
      frameLayout.setConstraints( inventoryLabel, constraints );
      this.add( inventoryLabel );

      
      _inventoryTextArea.setEditable( false );
      _inventoryTextArea.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 4;
      constraints.gridwidth = 3;
      constraints.gridheight = 1;
      frameLayout.setConstraints( _inventoryTextArea, constraints );
      this.add( _inventoryTextArea );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------

      Label accountLabel = new Label( ACCOUNT_LABEL, Label.LEFT );
      accountLabel.setFont( buttonFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 5;
      constraints.gridwidth = 1;
      constraints.anchor = GridBagConstraints.WEST;
      frameLayout.setConstraints( accountLabel, constraints );
      this.add( accountLabel );
      
      _accountTextField.setEditable( false );
      _accountTextField.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 1;
      constraints.gridy = 5;
      constraints.gridwidth = GridBagConstraints.RELATIVE;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      frameLayout.setConstraints( _accountTextField, constraints );
      this.add( _accountTextField );
      

      Button quitButton = new Button( QUIT_BUTTON_LABEL );
      quitButton.setFont( buttonFont );
      quitButton.setActionCommand( QUIT_BUTTON_LABEL );
      quitButton.addActionListener( this );
      constraints = new GridBagConstraints();
      constraints.gridx = 3;
      constraints.gridy = 5;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.anchor = GridBagConstraints.SOUTHEAST;
      frameLayout.setConstraints( quitButton, constraints );
      this.add( quitButton );

      // Get the current host using the static method on the BuyerFrame.
      String currentHost = BuyerFrame.getCurrentHostName();
      String frameTitle = null;
      if( _buyerName != null )
      {
         frameTitle = _buyerName + " agent on " + currentHost;
      }
      else
      {
         frameTitle = "Agent on " + currentHost;
      }
      this.setTitle( frameTitle );

      // Add a window adapter to listen for a signal from the window manager,
      // in case the user chooses Close from the menu.
      WindowAdapter closeWindowAdapter = new WindowAdapter()
      {
         public void windowClosing( WindowEvent windowEvent )
         {
            dispose();
         }
      };
      this.addWindowListener( closeWindowAdapter );
      
      // Invoke superclass methods Window::pack and Window::show
      this.pack();
      this.show();   
   }


/**
  * Description : 
  * 
  *   
  * @param 
  * @return Object  
  * @exception 
  * @author Jim Donohoe
  */
   public Object clone()
   {
      // For this object, use a shallow clone. 
      return this;
   }


/**
  * Description : Build a KQML message and send it to the parent agent. 
  * BuyerCallback is registered as an action listener for the two buttons
  * on the interface, so this method will be invoked when either button gets
  * pushed.
  *  
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void actionPerformed( ActionEvent event )
   {
      //-----------------------------------------------------------------------
      // Instantiate an empty KQML message, then fill it in with the values
      // that are expected by the agent for each type of event.  For either
      // the print or quit command the KQML performative is "achieve".  The
      // message content will differ depending on the command.  Note that the
      // content of the message must match the value expected by the agent in
      // order for the message to have the desired effect.  
      //-----------------------------------------------------------------------
 
      KqmlMessage message = new KqmlMessage();

      if( event.getActionCommand().equals( SHOP_BUTTON_LABEL ))
      {
         if( _productChoice.getSelectedIndex() == 0 )
         {
            // Product not selected.
            return;
         }
         String productName = _productChoice.getSelectedItem();

         Product p = (Product) _productTable.get( productName );

         // Initialize the quantity with the default value.
         int quantity = 1;

         String quantityText = _quantityTextField.getText().trim();
         
         if( ! quantityText.equals( EMPTY_STRING ) )
         {
            quantity = Integer.parseInt( quantityText );
         }

         String unitName = p.getUnitName();

         // Create a new PriceQuote object which will get passed to the
         // buyer agent and from there forwarded on to the stores.
         PriceQuote priceQuote = new PriceQuote( productName, quantity,
                                                 unitName );

         message.setPerformative( "forward" );
         message.setContent( priceQuote );

         // Clear the user input area.
         _quantityTextField.setText( EMPTY_STRING );
      }
      else if( event.getActionCommand().equals( QUIT_BUTTON_LABEL ))
      {
         message.setPerformative( "achieve" );
         message.setContent( "Quit" );
      }
      
      //-----------------------------------------------------------------------
      // Send the message from this PAC object to the parent agent.  The
      // addressing and sending of the message is handled by the communication
      // system.
      //-----------------------------------------------------------------------

      _pacCommSystem.sendKqmlMessageToAgent( message );
   }


/**
  * Description : Returns the name of the current host machine. 
  * 
  * @param 
  * @return String
  * @author Jim Donohoe
  */
   public static String getCurrentHostName()
   {
      InetAddress inetAddress = null;
      String hostName = null;
      try
      {
         inetAddress = InetAddress.getLocalHost();
      }
      catch( UnknownHostException e )
      {
         System.out.println("getHostName UnknownHostException");
      }

      if( inetAddress != null )
      {
         hostName = inetAddress.getHostName();
      }
      return hostName;   
   }   
   
}

/*******************************************************************************
 End of : BuyerFrame.java
*******************************************************************************/
