/*******************************************************************************
* Copyright Reticular Systems Inc. 1997    All rights reserved.
* Proprietary Information, Property of Reticular Systems Inc.
* Filename : SellerFrame.java
*
* RCS: $Revision: 1.6 $  $Date: 1998/07/09 18:12:20 $  $Author: sandoval $ 
* Created by: Jim Donohoe -- Tue Sep  9 16:44:59 1997
*******************************************************************************/

package com.reticular.agents.buyerSeller;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.reticular.agentBuilder.agent.perception.*;

/*******************************************************************************
* Description : Frame provides a simple interface for the message testing
* agent. This interface has a scrollable text area, a print button, and a quit
* button.
*
* 
* @author Jim Donohoe
*******************************************************************************/
public class SellerFrame extends Frame implements Runnable, Cloneable
{ 

//PUBLIC DATA
   public static final int FRAME_WIDTH = 800;
   public static final int FRAME_HEIGHT = 600;
      
   public static final int INV_TEXT_AREA_ROWS = 8;
   public static final int INV_TEXT_AREA_COLS = 48;
   public static final int MSSG_TEXT_AREA_ROWS = 8;
   public static final int MSSG_TEXT_AREA_COLS = 48;
   public static final int QUANT_TEXT_FIELD_COLS = 8;
   public static final String EMPTY_STRING = "";

   public static final String INVENTORY_LABEL = "Inventory:";
   public static final String ACCOUNT_LABEL = "Account:";
   public static final int ACC_TEXT_FIELD_COLS = 32;

   public static final String LABEL_FONT_NAME = "SansSerif";
   public static final int LABEL_FONT_STYLE = Font.BOLD;
   public static final int LABEL_FONT_SIZE = 18;

   public static final String TEXT_FONT_NAME = "SansSerif";
   public static final int TEXT_FONT_STYLE = Font.PLAIN;
   public static final int TEXT_FONT_SIZE = 16;



//PRIVATE DATA
   private TextArea _messageTextArea;
   private TextArea _inventoryTextArea;
   private TextField _accountTextField;
   private int _numberOfProducts;
   private Hashtable _inventoryTable;
   private Hashtable _priceTable;
   private String _sellerName;

   private static final int MAX_PRODUCTS = 10;

   
/********************************PUBLIC METHODS********************************

                                 

/**
  * Description : SellerFrame constructor
  *  
  * @param sellerName - String
  * @param  
  * @return   
  * @exception 
  * @author Jim Donohoe
  */
   public SellerFrame( String sellerName )
   {
      _sellerName = sellerName;
      
      _inventoryTable = new Hashtable();
      _priceTable = new Hashtable();

      // Initialize the components which may be needed before the interface
      // is fully built.
      _inventoryTextArea = new TextArea( EMPTY_STRING,
                                         INV_TEXT_AREA_ROWS,
                                         INV_TEXT_AREA_COLS,
                                         TextArea.SCROLLBARS_VERTICAL_ONLY );
      
      _accountTextField = new TextField( EMPTY_STRING );
   }



/**
  * Description : Append the string argument to the text area.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayMessage( String string )
   {
      _messageTextArea.append( string + "\n");
   }


/**
  * Description : Append the string argument to the text area.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayInventory( Product product )
   {
      // We need to keep the previous records in an table, so that we can
      // show the whole inventory each time we update it with a new entry.

      String itemName = product.getName();
      
      _inventoryTable.remove( itemName );

      _inventoryTable.put( itemName, product );

      // Clear the display area.
      _inventoryTextArea.setText("");

      Enumeration records = _inventoryTable.elements();
      while( records.hasMoreElements() )
      {
         try
         {
            Product prod = (Product) records.nextElement();

            Float unitPrice = (Float) _priceTable.get( prod.getName() );

            String recordString = prod.getName() + ": " +
               Integer.toString( prod.getQuantity() ) + " " +
               prod.getUnitName() + "(s)   Price = " + unitPrice + "/" +
               prod.getUnitName() + "\n";

            _inventoryTextArea.append( recordString );
         }
         catch( NoSuchElementException e )
         {
         }
      }
   }      



/**
  * Description : Convert the float to a string and display it in the
  * text field.
  * 
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void displayAccount( float accountValue )
   {
      _accountTextField.setText( "$ " + Float.toString( accountValue ));
   }      
   
/**
  * Description : Adds an item to the choice
  * 
  *   
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void addProduct( Product product )
   {
      // Add a name-keyed entry with an inventory record value.  Initialize
      // the quantity with zero, it will get changed later.
      String productName = product.getName();
      _inventoryTable.put( productName, product );

      // Add an entry to the price table so we can find the unit price
      // of this product.
      _priceTable.put( productName, new Float(product.getUnitPrice()) );
   }
   
/**
  * Description : 
  * 
  *   
  * @param 
  * @return void  
  * @exception 
  * @author Jim Donohoe
  */
   public void run( )
   {
      //-----------------------------------------------------------------------
      //            Set up the frame layout and common fonts 
      //-----------------------------------------------------------------------
   
      GridBagLayout frameLayout = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      this.setLayout( frameLayout );

      Font textFont = new Font( TEXT_FONT_NAME, TEXT_FONT_STYLE,
                                TEXT_FONT_SIZE );

      Font labelFont = new Font( LABEL_FONT_NAME, LABEL_FONT_STYLE,
                                 LABEL_FONT_SIZE );

      Label sellerLabel = new Label( _sellerName, Label.LEFT );
      sellerLabel.setFont( labelFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 0;
      constraints.gridwidth = 2;
      constraints.anchor = GridBagConstraints.NORTHWEST;
      frameLayout.setConstraints( sellerLabel, constraints );
      this.add( sellerLabel );


      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------
      _messageTextArea = new TextArea( EMPTY_STRING,
                                       MSSG_TEXT_AREA_ROWS,
                                       MSSG_TEXT_AREA_COLS,
                                       TextArea.SCROLLBARS_VERTICAL_ONLY );

      _messageTextArea.setEditable( false );
      _messageTextArea.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 1;
      constraints.gridwidth = 4;
      constraints.anchor = GridBagConstraints.EAST;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      frameLayout.setConstraints( _messageTextArea, constraints );
      this.add( _messageTextArea );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------

      Label inventoryLabel = new Label( INVENTORY_LABEL, Label.LEFT );
      inventoryLabel.setFont( labelFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 2;
      constraints.anchor = GridBagConstraints.WEST;
      frameLayout.setConstraints( inventoryLabel, constraints );
      this.add( inventoryLabel );
      
      _inventoryTextArea.setEditable( false );
      _inventoryTextArea.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 3;
      constraints.gridwidth = 4;
      constraints.anchor = GridBagConstraints.EAST;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      frameLayout.setConstraints( _inventoryTextArea, constraints );
      this.add( _inventoryTextArea );

      //-----------------------------------------------------------------------
      // 
      //-----------------------------------------------------------------------

      Label accountLabel = new Label( ACCOUNT_LABEL, Label.LEFT );
      accountLabel.setFont( labelFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 0;
      constraints.gridy = 4;
      constraints.gridwidth = 1;
      constraints.anchor = GridBagConstraints.WEST;
      frameLayout.setConstraints( accountLabel, constraints );
      this.add( accountLabel );
      
      _accountTextField.setEditable( false );
      _accountTextField.setFont( textFont );
      constraints = new GridBagConstraints();
      constraints.gridx = 1;
      constraints.gridy = 4;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      frameLayout.setConstraints( _accountTextField, constraints );
      this.add( _accountTextField );


      // Get the current host using the static method on the BuyerFrame.
      // This will be the name of the machine this SellerFrame is running
      // on, not necessarily the same machine as the Buyer.
      String currentHost = BuyerFrame.getCurrentHostName();
      String frameTitle = null;
      if( _sellerName != null )
      {
         frameTitle = _sellerName +  " agent on " + currentHost;
      }
      else
      {
         frameTitle = "Agent on " + currentHost;
      }
      this.setTitle( frameTitle );

      // Add a window adapter to listen for a signal from the window manager,
      // in case the user chooses Close from the menu.
      WindowAdapter closeWindowAdapter = new WindowAdapter()
         {
            public void windowClosing( WindowEvent windowEvent )
            {
               dispose();
            }
         };
      this.addWindowListener( closeWindowAdapter );
      
      // Invoke superclass methods Window::pack and Window::show
      this.pack();
      this.show();   
   }


/**
  * Description : 
  * 
  *   
  * @param 
  * @return Object  
  * @exception 
  * @author Jim Donohoe
  */
   public Object clone()
   {
      // For this object, use a shallow clone. 
      return this;
   }


}

/*******************************************************************************
 End of : SellerFrame.java
*******************************************************************************/
